@@ -1,4 +1,6 @@
+import asyncio
 import asyncio.coroutines
+import contextvars
 import functools
 import inspect
 import os
@@ -9,15 +11,9 @@ import weakref
 from concurrent.futures import Future, ThreadPoolExecutor
 from typing import Any, Callable, Dict, Optional, overload
 
-from .compatibility import get_running_loop
 from .current_thread_executor import CurrentThreadExecutor
 from .local import Local
 
-if sys.version_info >= (3, 7):
-    import contextvars
-else:
-    contextvars = None
-
 
 def _restore_context(context):
     # Check for changes in contextvars, and set them to the current
@@ -55,8 +51,6 @@ class ThreadSensitiveContext:
     In Python 3.7+, the ThreadSensitiveContext() context manager may be used to
     specify a thread pool per context.
 
-    In Python 3.6, usage of this context manager has no effect.
-
     This context manager is re-entrant, so only the outer-most call to
     ThreadSensitiveContext will set the context.
 
@@ -70,32 +64,22 @@ class ThreadSensitiveContext:
     def __init__(self):
         self.token = None
 
-    if contextvars:
+    async def __aenter__(self):
+        try:
+            SyncToAsync.thread_sensitive_context.get()
+        except LookupError:
+            self.token = SyncToAsync.thread_sensitive_context.set(self)
 
-        async def __aenter__(self):
-            try:
-                SyncToAsync.thread_sensitive_context.get()
-            except LookupError:
-                self.token = SyncToAsync.thread_sensitive_context.set(self)
+        return self
 
-            return self
+    async def __aexit__(self, exc, value, tb):
+        if not self.token:
+            return
 
-        async def __aexit__(self, exc, value, tb):
-            if not self.token:
-                return
-
-            executor = SyncToAsync.context_to_thread_executor.pop(self, None)
-            if executor:
-                executor.shutdown()
-            SyncToAsync.thread_sensitive_context.reset(self.token)
-
-    else:
-
-        async def __aenter__(self):
-            return self
-
-        async def __aexit__(self, exc, value, tb):
-            pass
+        executor = SyncToAsync.context_to_thread_executor.pop(self, None)
+        if executor:
+            executor.shutdown()
+        SyncToAsync.thread_sensitive_context.reset(self.token)
 
 
 class AsyncToSync:
@@ -118,11 +102,17 @@ class AsyncToSync:
     # Local, not a threadlocal, so that tasks can work out what their parent used.
     executors = Local()
 
+    # When we can't find a CurrentThreadExecutor from the context, such as
+    # inside create_task, we'll look it up here from the running event loop.
+    loop_thread_executors: "Dict[asyncio.AbstractEventLoop, CurrentThreadExecutor]" = {}
+
     def __init__(self, awaitable, force_new_loop=False):
         if not callable(awaitable) or not _iscoroutinefunction_or_partial(awaitable):
             # Python does not have very reliable detection of async functions
             # (lots of false negatives) so this is just a warning.
-            warnings.warn("async_to_sync was passed a non-async-marked callable")
+            warnings.warn(
+                "async_to_sync was passed a non-async-marked callable", stacklevel=2
+            )
         self.awaitable = awaitable
         try:
             self.__self__ = self.awaitable.__self__
@@ -133,7 +123,7 @@ class AsyncToSync:
             self.main_event_loop = None
         else:
             try:
-                self.main_event_loop = get_running_loop()
+                self.main_event_loop = asyncio.get_running_loop()
             except RuntimeError:
                 # There's no event loop in this thread. Look for the threadlocal if
                 # we're inside SyncToAsync
@@ -152,7 +142,7 @@ class AsyncToSync:
     def __call__(self, *args, **kwargs):
         # You can't call AsyncToSync from a thread with a running event loop
         try:
-            event_loop = get_running_loop()
+            event_loop = asyncio.get_running_loop()
         except RuntimeError:
             pass
         else:
@@ -162,12 +152,9 @@ class AsyncToSync:
                     "just await the async function directly."
                 )
 
-        if contextvars is not None:
-            # Wrapping context in list so it can be reassigned from within
-            # `main_wrap`.
-            context = [contextvars.copy_context()]
-        else:
-            context = None
+        # Wrapping context in list so it can be reassigned from within
+        # `main_wrap`.
+        context = [contextvars.copy_context()]
 
         # Make a future for the return information
         call_result = Future()
@@ -182,6 +169,7 @@ class AsyncToSync:
             old_current_executor = None
         current_executor = CurrentThreadExecutor()
         self.executors.current = current_executor
+        loop = None
         # Use call_soon_threadsafe to schedule a synchronous callback on the
         # main event loop's thread if it's there, otherwise make a new loop
         # in this thread.
@@ -193,6 +181,7 @@ class AsyncToSync:
             if not (self.main_event_loop and self.main_event_loop.is_running()):
                 # Make our own event loop - in a new thread - and run inside that.
                 loop = asyncio.new_event_loop()
+                self.loop_thread_executors[loop] = current_executor
                 loop_executor = ThreadPoolExecutor(max_workers=1)
                 loop_future = loop_executor.submit(
                     self._run_event_loop, loop, awaitable
@@ -212,12 +201,13 @@ class AsyncToSync:
                     current_executor.run_until_future(call_result)
         finally:
             # Clean up any executor we were running
+            if loop is not None:
+                del self.loop_thread_executors[loop]
             if hasattr(self.executors, "current"):
                 del self.executors.current
             if old_current_executor:
                 self.executors.current = old_current_executor
-            if contextvars is not None:
-                _restore_context(context[0])
+            _restore_context(context[0])
 
         # Wait for results from the future.
         return call_result.result()
@@ -233,10 +223,7 @@ class AsyncToSync:
             try:
                 # mimic asyncio.run() behavior
                 # cancel unexhausted async generators
-                if sys.version_info >= (3, 7, 0):
-                    tasks = asyncio.all_tasks(loop)
-                else:
-                    tasks = asyncio.Task.all_tasks(loop)
+                tasks = asyncio.all_tasks(loop)
                 for task in tasks:
                     task.cancel()
 
@@ -297,8 +284,7 @@ class AsyncToSync:
         finally:
             del self.launch_map[current_task]
 
-            if context is not None:
-                context[0] = contextvars.copy_context()
+            context[0] = contextvars.copy_context()
 
 
 class SyncToAsync:
@@ -325,7 +311,9 @@ class SyncToAsync:
 
     # If they've set ASGI_THREADS, update the default asyncio executor for now
     if "ASGI_THREADS" in os.environ:
-        loop = get_running_loop()
+        # We use get_event_loop here - not get_running_loop - as this will
+        # be run at import time, and we want to update the main thread's loop.
+        loop = asyncio.get_event_loop()
         loop.set_default_executor(
             ThreadPoolExecutor(max_workers=int(os.environ["ASGI_THREADS"]))
         )
@@ -341,21 +329,15 @@ class SyncToAsync:
 
     # Maintain a contextvar for the current execution context. Optionally used
     # for thread sensitive mode.
-    if sys.version_info >= (3, 7):
-        thread_sensitive_context: "contextvars.ContextVar[str]" = (
-            contextvars.ContextVar("thread_sensitive_context")
-        )
-    else:
-        thread_sensitive_context: None = None
+    thread_sensitive_context: "contextvars.ContextVar[str]" = contextvars.ContextVar(
+        "thread_sensitive_context"
+    )
 
     # Contextvar that is used to detect if the single thread executor
     # would be awaited on while already being used in the same context
-    if sys.version_info >= (3, 7):
-        deadlock_context: "contextvars.ContextVar[bool]" = contextvars.ContextVar(
-            "deadlock_context"
-        )
-    else:
-        deadlock_context: None = None
+    deadlock_context: "contextvars.ContextVar[bool]" = contextvars.ContextVar(
+        "deadlock_context"
+    )
 
     # Maintaining a weak reference to the context ensures that thread pools are
     # erased once the context goes out of scope. This terminates the thread pool.
@@ -384,7 +366,7 @@ class SyncToAsync:
             pass
 
     async def __call__(self, *args, **kwargs):
-        loop = get_running_loop()
+        loop = asyncio.get_running_loop()
 
         # Work out what thread to run the code in
         if self._thread_sensitive:
@@ -405,6 +387,9 @@ class SyncToAsync:
                     # Create new thread executor in current context
                     executor = ThreadPoolExecutor(max_workers=1)
                     self.context_to_thread_executor[thread_sensitive_context] = executor
+            elif loop in AsyncToSync.loop_thread_executors:
+                # Re-use thread executor for running loop
+                executor = AsyncToSync.loop_thread_executors[loop]
             elif self.deadlock_context and self.deadlock_context.get(False):
                 raise RuntimeError(
                     "Single thread executor already being used, would deadlock"
@@ -418,34 +403,32 @@ class SyncToAsync:
             # Use the passed in executor, or the loop's default if it is None
             executor = self._executor
 
-        if contextvars is not None:
-            context = contextvars.copy_context()
-            child = functools.partial(self.func, *args, **kwargs)
-            func = context.run
-            args = (child,)
-            kwargs = {}
-        else:
-            func = self.func
+        context = contextvars.copy_context()
+        child = functools.partial(self.func, *args, **kwargs)
+        func = context.run
+        args = (child,)
+        kwargs = {}
 
-        # Run the code in the right thread
-        future = loop.run_in_executor(
-            executor,
-            functools.partial(
-                self.thread_handler,
-                loop,
-                self.get_current_task(),
-                sys.exc_info(),
-                func,
-                *args,
-                **kwargs,
-            ),
-        )
-        ret = await asyncio.wait_for(future, timeout=None)
+        try:
+            # Run the code in the right thread
+            future = loop.run_in_executor(
+                executor,
+                functools.partial(
+                    self.thread_handler,
+                    loop,
+                    self.get_current_task(),
+                    sys.exc_info(),
+                    func,
+                    *args,
+                    **kwargs,
+                ),
+            )
+            ret = await asyncio.wait_for(future, timeout=None)
 
-        if contextvars is not None:
+        finally:
             _restore_context(context)
-        if self.deadlock_context:
-            self.deadlock_context.set(False)
+            if self.deadlock_context:
+                self.deadlock_context.set(False)
 
         return ret
 
@@ -491,17 +474,11 @@ class SyncToAsync:
     @staticmethod
     def get_current_task():
         """
-        Cross-version implementation of asyncio.current_task()
-
-        Returns None if there is no task.
+        Implementation of asyncio.current_task()
+        that returns None if there is no task.
         """
         try:
-            if hasattr(asyncio, "current_task"):
-                # Python 3.7 and up
-                return asyncio.current_task()
-            else:
-                # Python 3.6
-                return asyncio.Task.current_task()
+            return asyncio.current_task()
         except RuntimeError:
             return None
 
